/*
 * Copyright (c) 2001-2010, Purdue University
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright
 *  *    notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of the Purdue University nor the
 *    names of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package immortal;

import java.io.DataOutputStream;
import java.io.IOException;
import java.lang.Math;

/**
 * Cyclic buffer of frames. Used for frames generated by simulator, but not yet processed by detector.
 * The memory is allocated at construction time and always re-used (frame data is copied in the same locations).
 * The frames, as well as the buffer, live in immortal memory. The constructor runs in immortal memory.
 * Note that the buffer is (intentionally) not synchronized. In a weird combination of priorities that is not
 * intended to be used, buffer frames could be overwritten.
 */
public class FrameBuffer {
	
	// empty buffer ... first == last
	// full buffer .... last + 1 == first
	//    - so there is still one empty slot, but we don't want to use it,
	//      because we would not then recognize empty from full buffer
	//
	// last .. where the next frame will be stored
	// first .. where the next frame will be read 

	public int first, last;
	public RawFrame[] frames;
    protected float t;
    protected int[] lengths; 
    protected byte[] callsigns; 
    
    public FrameBuffer() {
    
        t=0.0f;
        lengths= new int[Constants.NUMBER_OF_PLANES];
        callsigns=new byte[Constants.NUMBER_OF_PLANES*6];
        for (int k=0;k<lengths.length;k++) lengths[k]=6;
	}
	
	public void putFrameInternal(final float[] positions_, final int[] lengths_, final byte[] callsigns_) {
	}

	static int frameno = 0;

	public void putFrame(final float[] positions_, final int[] lengths_, final byte[] callsigns_) {
	}

        

	public RawFrame getFrame() {
        RawFrame result=new RawFrame();
        for (byte k=0;k<Constants.NUMBER_OF_PLANES;k++) {
            callsigns[6*k]=112;
            callsigns[6*k+1]=108;
            callsigns[6*k+2]=97;
            callsigns[6*k+3]=110;
            callsigns[6*k+4]=101;
            callsigns[6*k+5]=(byte)(49+k);
        }
        float positions[] = new float[60*3];
        
        for (int k=0;k<Constants.NUMBER_OF_PLANES/2;k++) {
            positions[3*k]=(float)(400*Math.cos(t)+500+10*k);
            positions[3*k+1]=100.0f;
            positions[3*k+2]=5.0f;
            positions[Constants.NUMBER_OF_PLANES/2*3+3*k]=(float)(400*Math.sin(t)+500+10*k);
            positions[Constants.NUMBER_OF_PLANES/2*3+3*k+1]=100.0f;
            positions[Constants.NUMBER_OF_PLANES/2*3+3*k+2]=5.0f;
        }
        // increase the time
        t=t+0.25f;
        result.copy(lengths,callsigns,positions);
        return result;
       
	}
}
